@@ -1,6 +1,7 @@
 #![deny(clippy::all)]
 #![deny(missing_docs)]
 #![forbid(unsafe_code)]
+#![doc(html_root_url = "https://docs.rs/libunftp/0.18.0")]
 
 //! libunftp is an extensible, async, cloud orientated FTP(S) server library.
 //!
@@ -19,8 +20,8 @@
 //!
 //! ```toml
 //! [dependencies]
-//! libunftp = "0.17.4"
-//! unftp-sbe-fs = "0.1"
+//! libunftp = "0.18.0"
+//! unftp-sbe-fs = "0.2.0"
 //! tokio = { version = "1", features = ["full"] }
 //! ```
 //! Now you're ready to develop your server! Add the following to src/main.rs:
