@@ -28,6 +28,7 @@ use futures::{
     channel::mpsc::{channel, Receiver, Sender},
     SinkExt, StreamExt,
 };
+use rustls::{ServerSession, Session as RustlsSession};
 use std::{net::SocketAddr, ops::Range, sync::Arc, time::Duration};
 use tokio::{
     io::{AsyncRead, AsyncWrite},
@@ -60,7 +61,7 @@ where
     pub sitemd5: SiteMd5,
 }
 
-/// Does TCP processing when a FTP client connects
+/// Does TCP processing when an FTP client connects
 #[tracing_attributes::instrument]
 pub async fn spawn<Storage, User>(
     config: Config<Storage, User>,
@@ -105,7 +106,7 @@ where
     let event_chain = PrimaryEventHandler {
         logger: logger.clone(),
         session: shared_session.clone(),
-        authenticator,
+        authenticator: authenticator.clone(),
         tls_configured,
         passive_ports,
         passive_host,
@@ -200,7 +201,14 @@ where
                         };
                         let accepted = acceptor.accept(io).await;
                         let io: Box<dyn AsyncReadAsyncWriteSendUnpin> = match accepted {
-                            Ok(stream) => Box::new(stream),
+                            Ok(stream) => {
+                                let s: &ServerSession = stream.get_ref().1;
+                                if let Some(certs) = s.get_peer_certificates() {
+                                    let mut session = shared_session.lock().await;
+                                    session.cert_chain = Some(certs.iter().map(|c| crate::auth::ClientCert(c.0.clone())).collect());
+                                }
+                                Box::new(stream)
+                            }
                             Err(err) => {
                                 slog::warn!(logger, "Closing control channel. Could not upgrade to TLS: {}", err);
                                 return;
@@ -343,7 +351,11 @@ where
                 session.state = WaitCmd;
                 Ok(Reply::new(ReplyCode::UserLoggedIn, "User logged in, proceed"))
             }
-            AuthFailed => Ok(Reply::new(ReplyCode::NotLoggedIn, "Authentication failed")),
+            AuthFailed => {
+                let mut session = self.session.lock().await;
+                session.state = New; // According to RFC 959, a PASS command MUST precede a USER command
+                Ok(Reply::new(ReplyCode::NotLoggedIn, "Authentication failed"))
+            }
             StorageError(error_type) => match error_type.kind() {
                 ErrorKind::ExceededStorageAllocationError => Ok(Reply::new(ReplyCode::ExceededStorageAllocation, "Exceeded storage allocation")),
                 ErrorKind::FileNameNotAllowedError => Ok(Reply::new(ReplyCode::BadFileName, "File name not allowed")),
